###########################################################################
# File: heritability_examples.R
# Author: Wonil Chung, Vered Madar
# First Revised: 11.10.2012
# Last Revised:  07.17.2012
# Description: Example code for estimating heritability
###########################################################################

### Set starting and ending index
trans.start = 1; trans.end = 2

### Set Working Directory
setwd("/nas02/home/w/c/wchung11/heritability/godot/NTR_final_expression_twins/")
source("optim_heritability.R")

### Read Y (expression data)
RNA=read.csv("RNA_INQ_NTR_2752.csv",header=T) # expression data
dim(RNA) #[1] 47678  2753
y.all.matrix=RNA[,-1]
transcripts=RNA[,1]

y.all.matrix=as.matrix(y.all.matrix)
dim(y.all.matrix)
n.transcripts=dim(y.all.matrix)[1]
n.all.sub=dim(y.all.matrix)[2]

### Set number of twins
#DZ pairs, MZ pairs and singletons
n.DOS=252       # DZ pairs of male and female (twins)
n.female.DZ=256 # DZ pairs females 
n.male.DZ=110   # DZ pairs males 
n.female.MZ=481 # MZ pairs females
n.male.MZ=209   # MZ pairs males
n.all.s=136     # singletons

#counts for the whole set of twins
n.all.DZ=n.female.DZ+n.male.DZ+n.DOS #DZ pairs = 618
n.all.MZ=n.female.MZ+n.male.MZ       #MZ pairs = 714

#set MZ DZ pairs and singletons
yDZ = y.all.matrix[,1:(n.all.DZ*2)]
yMZ = y.all.matrix[,(n.all.DZ*2+1):(n.all.DZ*2+n.all.MZ*2)]
ysingle=y.all.matrix[,(2*n.all.DZ+2*n.all.MZ+1):n.all.sub]
dim(yDZ)
dim(yMZ)
dim(ysingle)

#set X (covariate data)
X = as.matrix(read.csv("ntr_covariate_final_pc_qnorm.csv"))
xDZ = X[1:(n.all.DZ*2),]
xMZ = X[(n.all.DZ*2+1):(n.all.DZ*2+n.all.MZ*2),]
xsingle = X[(2*n.all.DZ+2*n.all.MZ+1):n.all.sub,]
dim(xDZ)
dim(xMZ)
dim(xsingle)


### Conduct Simulations
n.transcripts = trans.end-trans.start+1
beta.matrix = matrix(0, ncol(xMZ), n.transcripts)
sigma.matrix = matrix(0, 1, n.transcripts)

h.matrix = matrix(0, 2, n.transcripts)
wald.matrix = matrix(0, 3, n.transcripts)
pvalue.matrix = matrix(0, 3, n.transcripts)

t.matrix = matrix(0, 2, n.transcripts)
pvalue.t.matrix = matrix(0, 4, n.transcripts)

for (i in (trans.start:trans.end)){
 print(i)

 # Calcuate heritability using optim function
 out=estimate.h2.cov.optim(yMZ1=matrix(yMZ[i,],1,n.all.MZ*2),yDZ1=matrix(yDZ[i,],1,n.all.DZ*2),
    ysingle1=matrix(ysingle[i,],1,n.all.s) ,xMZ1=xMZ,xDZ1=xDZ,xsingle1=xsingle)
    
 # Estimate beta, sigma
 beta.matrix[,i-(trans.start-1)] = out$beta.hat
 sigma.matrix[,i-(trans.start-1)] = out$sigma.hat
 
 # Estimate a2 and c2 estimate
 h.matrix[,i-(trans.start-1)] = matrix(c(2,-1,-2,2),2,2)%*%t(out$theta.hat)
 m = matrix(c(2,-1,-2,2) ,2,2)
 h.var = m%*%solve(matrix(out$hessian.hat,2,2))%*%t(m)

 # Calculate p-value of H0: a2>0, a2<0, c2>0 and c2<0
 t.matrix[1,i-(trans.start-1)] = h.matrix[1,i-(trans.start-1)]/sqrt(h.var[1,1])
 t.matrix[2,i-(trans.start-1)] = h.matrix[2,i-(trans.start-1)]/sqrt(h.var[2,2])

 pvalue.t.matrix[1,i-(trans.start-1)] = pt(t.matrix[1,i-(trans.start-1)],n.all.sub-1,lower.tail=F)
 pvalue.t.matrix[2,i-(trans.start-1)] = pt(t.matrix[1,i-(trans.start-1)],n.all.sub-1,lower.tail=T)
 pvalue.t.matrix[3,i-(trans.start-1)] = pt(t.matrix[2,i-(trans.start-1)],n.all.sub-1,lower.tail=F)
 pvalue.t.matrix[4,i-(trans.start-1)] = pt(t.matrix[2,i-(trans.start-1)],n.all.sub-1,lower.tail=T)

}

# Save Results
setwd("/lustre/scr/w/c/wchung11/heritabillity/final_qnorm")

string = paste("_heritability_final_",trans.start,"_",trans.end,sep="")

rowMeans(h.matrix)
rowMeans(beta.matrix)
rowMeans(sigma.matrix)

write.csv(h.matrix, paste("h",string,".csv",sep=""), row.names=F)
write.csv(beta.matrix, paste("beta",string,".csv",sep=""), row.names=F)
write.csv(sigma.matrix, paste("sigma",string,".csv",sep=""), row.names=F)
write.csv(t.matrix, paste("t",string,".csv",sep=""), row.names=F)
write.csv(pvalue.t.matrix, paste("pvalue_t",string,".csv",sep=""), row.names=F)




