###########################################################################
# File: optim_heritability.R
# Author: Wonil Chung, Vered Madar
# First Revised: 11.10.2012
# Last Revised:  07.17.2012
# R Functions: estimate.theta.cov, loglike.h2.cov, estimate.h2.cov.optim
# Description: R functions for estimating heritability
###########################################################################

###############################################################
##  Estimate beta and sigma using rho_mz and rho_dz
###############################################################
estimate.theta.cov=function(theta, yMZ1, yDZ1, ysingle1, xMZ1, xDZ1, xsingle1){
 
 n.transcripts=nrow(yMZ1)
 p=dim(as.matrix(xMZ1))[2]  #no of parametrs
 n.MZ=dim(yMZ1)[2]
 n.DZ=dim(yDZ1)[2]
 nsub=dim(yDZ1)[2]+dim(yMZ1)[2]+dim(ysingle1)[2]

 rho.MZ=theta[1]
 rho.DZ=theta[2]
 
 det.V.DZ=1-rho.DZ*rho.DZ
 det.V.MZ=1-rho.MZ*rho.MZ
 det.V=n.DZ/2*log(det.V.DZ)+n.MZ/2*log(det.V.MZ)

 inverse.V.DZ=matrix(c(1,-rho.DZ,-rho.DZ,1),2,2)/det.V.DZ
 inverse.V.MZ=matrix(c(1,-rho.MZ,-rho.MZ,1),2,2)/det.V.MZ
 inverse.V.DZ=diag(n.DZ/2)%x%inverse.V.DZ
 inverse.V.MZ=diag(n.MZ/2)%x%inverse.V.MZ

 XtX=t(xDZ1)%*%inverse.V.DZ%*%xDZ1+t(xMZ1)%*%inverse.V.MZ%*%xMZ1
 XtY=t(xDZ1)%*%inverse.V.DZ%*%t(yDZ1)+t(xMZ1)%*%inverse.V.MZ%*%t(yMZ1)
 
 if (length(ysingle1)>0) { 
      XtX=XtX+t(xsingle1)%*%xsingle1
      XtY=XtY+t(xsingle1)%*%t(ysingle1)
 }
 beta=solve(XtX)%*%XtY
 
 rDZ=t(yDZ1)-xDZ1%*%beta
 rMZ=t(yMZ1)-xMZ1%*%beta
 rtr=rep(0,n.transcripts)

 DZ.count=n.DZ/2
 first.DZ=2*c(1:DZ.count)-1
 second.DZ=first.DZ+1
 rDZ1=rDZ[first.DZ,]
 rDZ2=rDZ[second.DZ,] 

 MZ.count=n.MZ/2
 first.MZ=2*c(1:MZ.count)-1
 second.MZ=first.MZ+1
 rMZ1=rMZ[first.MZ,]
 rMZ2=rMZ[second.MZ,] 

 rtr = apply(matrix(rDZ1^2+rDZ2^2-2*rho.DZ*rDZ1*rDZ2,nrow=nrow(yDZ1)),1,sum)/det.V.DZ + 
     apply(matrix(rMZ1^2+rMZ2^2-2*rho.MZ*rMZ1*rMZ2,nrow=nrow(yMZ1)),1,sum)/det.V.MZ

 rsingle=0
 if (length(ysingle1)>0) {
    rsingle=t(ysingle1)-xsingle1%*%beta 
 }

 rtr=rtr+apply(rsingle^2,2,sum)
 sigma.hat=matrix(rtr/(nsub-p),n.transcripts,1)

 return(list(beta.hat=beta,sigma.hat=sigma.hat))

}


###############################################################
##  Reml log-likelihood function of rho_mz and rho_dz
###############################################################
loglike.h2.cov=function(theta, yMZ1, yDZ1, ysingle1, xMZ1, xDZ1, xsingle1){
 
 n.transcripts=nrow(yMZ1)
 p=dim(as.matrix(xMZ1))[2]  #no of parametrs
 n.MZ=dim(yMZ1)[2]
 n.DZ=dim(yDZ1)[2]
 nsub=dim(yDZ1)[2]+dim(yMZ1)[2]+dim(ysingle1)[2]

 rho.MZ=theta[1]
 rho.DZ=theta[2]
 
 det.V.DZ=1-rho.DZ*rho.DZ
 det.V.MZ=1-rho.MZ*rho.MZ
 det.V=n.DZ/2*log(det.V.DZ)+n.MZ/2*log(det.V.MZ)

 inverse.V.DZ=matrix(c(1,-rho.DZ,-rho.DZ,1),2,2)/det.V.DZ
 inverse.V.MZ=matrix(c(1,-rho.MZ,-rho.MZ,1),2,2)/det.V.MZ
 inverse.V.DZ=diag(n.DZ/2)%x%inverse.V.DZ
 inverse.V.MZ=diag(n.MZ/2)%x%inverse.V.MZ

 XtX=t(xDZ1)%*%inverse.V.DZ%*%xDZ1+t(xMZ1)%*%inverse.V.MZ%*%xMZ1
 XtY=t(xDZ1)%*%inverse.V.DZ%*%t(yDZ1)+t(xMZ1)%*%inverse.V.MZ%*%t(yMZ1)
 
 if (length(ysingle1)>0) { 
      XtX=XtX+t(xsingle1)%*%xsingle1
      XtY=XtY+t(xsingle1)%*%t(ysingle1)
 }
 beta=solve(XtX)%*%XtY
 
 rDZ=t(yDZ1)-xDZ1%*%beta
 rMZ=t(yMZ1)-xMZ1%*%beta
 rtr=rep(0,n.transcripts)

 DZ.count=n.DZ/2
 first.DZ=2*c(1:DZ.count)-1
 second.DZ=first.DZ+1
 rDZ1=rDZ[first.DZ,]
 rDZ2=rDZ[second.DZ,] 

 MZ.count=n.MZ/2
 first.MZ=2*c(1:MZ.count)-1
 second.MZ=first.MZ+1
 rMZ1=rMZ[first.MZ,]
 rMZ2=rMZ[second.MZ,] 

 rtr = apply(matrix(rDZ1^2+rDZ2^2-2*rho.DZ*rDZ1*rDZ2,nrow=nrow(yDZ1)),1,sum)/det.V.DZ +
    apply(matrix(rMZ1^2+rMZ2^2-2*rho.MZ*rMZ1*rMZ2,nrow=nrow(yMZ1)),1,sum)/det.V.MZ

 rsingle=0
 if (length(ysingle1)>0) {
    rsingle=t(ysingle1)-xsingle1%*%beta 
 }

 rtr=rtr+apply(rsingle^2,2,sum)
 
 ### For REML log-likelihood #####
 det.XtX = log(det( (t(xMZ1)%*%inverse.V.MZ%*%xMZ1+t(xDZ1)%*%inverse.V.DZ%*%xDZ1+t(xsingle1)%*%xsingle1)/(rtr/(nsub-p)) ))
 #################################

 loglike=0.5*(det.XtX+det.V+nsub*log(rtr/(nsub-p)))

 return(loglike)

}

###############################################################
##  Estimate rho_mz and rho_dz using optim function
###############################################################
estimate.h2.cov.optim=function(yMZ1,yDZ1,ysingle1,xMZ1,xDZ1,xsingle1){

 n.transcripts=nrow(yMZ1)

 theta=matrix(0,n.transcripts,2)
 hessian=matrix(0,n.transcripts,4)
 rho1.start = rho2.start = 0
 
 for (i in 1:n.transcripts){
    result=optim(par=c(rho1.start,rho2.start),
          fn=loglike.h2.cov,yMZ1=matrix(yMZ1[i,],nrow=1),yDZ1=matrix(yDZ1[i,],nrow=1),ysingle1=matrix(ysingle1[i,],nrow=1),
                    xMZ1=xMZ1,xDZ1=xDZ1,xsingle1=xsingle1, lower=-0.96, upper=0.96, hessian=T, method="L-BFGS-B")
    theta[i,] = result$par
    hessian[i,1:2] = result$hessian[1,]
    hessian[i,3:4] = result$hessian[2,]
 }

 # Estimate beta hat and sigma hat
 beta.hat1 = matrix(0, ncol(xMZ1), n.transcripts)
 sigma.hat1 = matrix(0, 1, n.transcripts)

 for (i in 1:n.transcripts){
  the = estimate.theta.cov(theta=theta[i,]
	      ,yMZ1=matrix(yMZ1[i,],nrow=1),yDZ1=matrix(yDZ1[i,],nrow=1),ysingle1=matrix(ysingle1[i,],nrow=1)
        ,xMZ1=xMZ1,xDZ1=xDZ1,xsingle1=xsingle1)
    beta.hat1[,i] = the$beta.hat 
    sigma.hat1[,i] = the$sigma.hat
 }

 return(list(theta.hat=theta,hessian.hat=hessian,beta.hat=beta.hat1,sigma.hat=sigma.hat1))

}


